﻿using UnityEngine;
using UnityEditor;
using System.Collections.Generic;
using System.IO;
using SimpleJSON;

public static class TexturePackerImport
{
    [MenuItem("Assets/Sprite Sheet Packer/Process to Sprites")]
    static void ProcessToSprite()
    {
        TextAsset txt = (TextAsset)Selection.activeObject;

        string rootPath = Path.GetDirectoryName(AssetDatabase.GetAssetPath(txt));
        string path = rootPath + "/" + txt.name + ".PNG";
        
        //To get the height of the image
        Texture2D tmpTexture = new Texture2D(1, 1);
        byte[] tmpBytes = File.ReadAllBytes(path);
        tmpTexture.LoadImage(tmpBytes);
        

        //Parses Sprite Sheet Packer files 
        List<SpriteMetaData> sprites = Parse(txt, tmpTexture.height);
        TextureImporter texImp = AssetImporter.GetAtPath(path) as TextureImporter;

        //Fetch old data if it exists
        Dictionary<string, SpriteMetaData> oldData = new Dictionary<string, SpriteMetaData>();
        for (int i = 0; i < texImp.spritesheet.Length; i++)
        {
            SpriteMetaData d = new SpriteMetaData();
            d.name = texImp.spritesheet[i].name;
            d.pivot = texImp.spritesheet[i].pivot;
            d.border = texImp.spritesheet[i].border;
            oldData.Add(d.name, d);
        }

        //Assing old border and pivot values to new sprite
        for (int i = 0; i < sprites.Count; i++)
        {
            SpriteMetaData it = sprites[i];
            if (oldData.ContainsKey(it.name))
            {
                SpriteMetaData old = oldData[it.name];
                it.pivot = new Vector2(old.pivot.x, old.pivot.y);
                it.border = new Vector4(old.border.x, old.border.y, old.border.z, old.border.w);
                sprites[i] = it;
            }
        }

        texImp.spritesheet = sprites.ToArray();
        texImp.textureType = TextureImporterType.Sprite;
        texImp.spriteImportMode = SpriteImportMode.Multiple;
        AssetDatabase.ImportAsset(path, ImportAssetOptions.ForceUpdate);

    }
    static List<SpriteMetaData> Parse(TextAsset file, int height)
    {
        List<SpriteMetaData> Sprites = new List<SpriteMetaData>();
        JSONNode sheet = JSONNode.Parse(file.text);
        Debug.Log("Parsing..");

        JSONArray jsArr = sheet["frames"].AsArray;

        Debug.Log(jsArr.Count);
        for (int i = 0; i < jsArr.Count; i++)
        {
            JSONNode spriteJS = jsArr[i];

            SpriteMetaData smd = new SpriteMetaData();

            string name = spriteJS["filename"];
            smd.name = name.Trim();

            Vector2 pivot = Vector2.zero;
            pivot.x = spriteJS["pivot"]["x"].AsFloat;
            pivot.y = spriteJS["pivot"]["y"].AsFloat;
            smd.pivot = pivot;
            
            int posx = spriteJS["frame"]["x"].AsInt;
            int posy = spriteJS["frame"]["y"].AsInt;
            int w = spriteJS["frame"]["w"].AsInt;
            int h = spriteJS["frame"]["h"].AsInt;

            //0,0 is top-left in unity
            posy = height - posy - h;

            smd.rect = new Rect(posx, posy, w, h);
            
            Sprites.Add(smd);
        }
        
        return Sprites;
    }
}