﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AmebaAnimBehavior : MonoBehaviour
{
    public Sprite IdleFace;
    public Sprite HurtFace;
    public Sprite DangerFace;
    public Sprite DeadFace;
  
    float stateTimer = 0.0f;
    
    enum faceEnum { idle, happy, hurt, dead};
    faceEnum AnimState = faceEnum.idle;
    
    private SpriteRenderer _face;
    public SpriteRenderer Face
    {
        get
        {
            if (_face == null)
                _face = GetComponent<SpriteRenderer>();

            return _face;
        }
    }
    private EyeBehavior[] _eyes;
    public EyeBehavior[] Eyes
    {
        get
        {
            if (_eyes == null)
                _eyes = GetComponentsInChildren<EyeBehavior>();

            return _eyes;
        }
    }
    
    public void Update()
    {
        if(AnimState == faceEnum.hurt)
        {
            stateTimer -= Time.deltaTime;
            if (stateTimer < 0.0f)
                SetIdle(true);
        }
    }
    
    public void SetIdle(bool always = false)
    {
        if (!always && AnimState == faceEnum.hurt)
            return;

        AnimState = faceEnum.idle;
        if (Face && IdleFace)
            Face.sprite = IdleFace;

        for (int i = 0; i < Eyes.Length; i++)
            Eyes[i].Visible(true);
    }

    public void TakingDamage()
    {
        stateTimer = 1.0f;
        AnimState = faceEnum.hurt;

        if (Face && HurtFace)
            Face.sprite = HurtFace;

        for (int i = 0; i < Eyes.Length; i++)
            Eyes[i].Visible(false);
    }
    
    public void Die()
    {
        AnimState = faceEnum.dead;
        if (Face && DeadFace)
            Face.sprite = DeadFace;

        for (int i = 0; i < Eyes.Length; i++)
            Eyes[i].Visible(false);
    }

    public void LookAt(Transform target)
    {
        if (Eyes == null || Eyes.Length == 0)
            return;

        for (int i = 0; i < Eyes.Length; i++)
        {
            Eyes[i].LookAt(target);
        }

        if(AnimState == faceEnum.idle)
        {
            if (target != null)
            {
                if (Face && DangerFace)
                    Face.sprite = DangerFace;
            }
            else
            {
                if (Face && IdleFace)
                    Face.sprite = IdleFace;
            }
        }
            
    }
}
