﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


public class EnemyBehavior : MoverBehavior
{
    public enum AITypeEnum { Agressive, Passive, Flee }
    public AITypeEnum AIType = AITypeEnum.Agressive;
    public bool DieOnHit = true;
    public int Damage = 1;

    public float LineOfSight = 3.0f;
    
    void Awake()
    {
        ChangeState(new EnemyIdleState());
    }

    private SpawnableBehavior _sb;

    public SpawnableBehavior SB
    {
        get
        {
            if (_sb == null)
                _sb = GetComponent<SpawnableBehavior>();

            return _sb;
        }
    }


    //Fixed state machine
    BaseState<EnemyBehavior> _currentState;
    public void ChangeState(BaseState<EnemyBehavior> newState)
    {
        if (_currentState != null)
            _currentState.Exit(this);

        _currentState = newState;
        _currentState.Enter(this);
    }

    void Update()
    {
        if (_currentState != null)
            _currentState.Update(this);
    }

    protected override void FixedUpdate()
    {
        base.FixedUpdate();

        if (_currentState != null)
            _currentState.FixedUpdate(this);
    }
}
