﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

public class GameManagerBehavior : MonoBehaviour, IObserver<CharacterEventArgs>, ISubject<GameEventArgs>
{
    public static GameManagerBehavior curret;
    public bool isRunning { get; private set; }

    public GameObject PlayerPrefab;
    public GameObject bacteriaPrefab;

    public CharacterBehavior Player { get; private set; }
    private GameHUD _hud;
    public GameHUD Hud
    {
        get
        {
            if (_hud == null)
                _hud = FindObjectOfType<GameHUD>();

            return _hud;
        }
    }
    private SmoothCamera2D _cam;
    public SmoothCamera2D CAM
    {
        get
        {
            if (_cam == null)
                _cam = FindObjectOfType<SmoothCamera2D>();

            return _cam;
        }
    }
    public int Score { get; private set; }

    public void Awake()
    {
        Init();
        curret = this;
    }

    public void Reset()
    {
        isRunning = true;
        Player.transform.position = Vector3.zero;
        Player.Reset();
        Score = 0;
    }

    void Start()
    {
        isRunning = true;
        Score = 0;
    }

    void Init()
    {
        if(PlayerPrefab != null)
        {
            GameObject go = Instantiate(PlayerPrefab, Vector3.zero, Quaternion.identity);
            Player = go.GetComponent<CharacterBehavior>();

            Player.Attach(this); //Listen to player

            if (Hud)
                Player.Attach(Hud); //make Hud listen to player
            else
                Debug.LogError("Hud not found");

            if (CAM)
            {
                CAM.target = Player.transform;
                Player.Attach(CAM);
            }
            else
                Debug.LogError("Smooth follow Camera missing");

            Player.Reset();
        }
        else
        {
            Debug.LogError("PlayerPrefab not found!");
        }
    }

    #region EventHandling

    public void HandleEvent(object sender, CharacterEventArgs e)
    {
        if(e.EventType == CharacterEventType.Died && isRunning)
        {
            isRunning = false;
            UIManager ui = FindObjectOfType<UIManager>();
            if(ui != null)
            {
                ui.GameOverW.Show();
            }
        }
    }
    
    public event EventHandler<GameEventArgs> Events;
    public void Attach(IObserver<GameEventArgs> observer)
    {
        Events += observer.HandleEvent;
    }

    public void Detach(IObserver<GameEventArgs> observer)
    {
        Events -= observer.HandleEvent;
    }

    #endregion
}

public enum GameEvents
{
    GameOver, ScoreUpdate
}

public class GameEventArgs : EventArgs
{
    public GameEvents EventType { get; private set; }
    public GameEventArgs(GameEvents evnt)
    {
        EventType = evnt;
    }
}