﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Rigidbody2D))]
public class MoverBehavior : MonoBehaviour
{
    public float MaxSpeed = 3.0f;
    public float velocityChange = 1.0f;
    public bool RotateTowardsDir = false;

    protected Rigidbody2D _rb;
    public Rigidbody2D RB
    {
        get
        {
            if(_rb == null)
            {
                _rb = GetComponent<Rigidbody2D>();
            }
            return _rb;
        }
    }

    protected Vector2 dirInput = Vector2.zero;
    public virtual void Move(Vector2 dir)
    {
        //Change movement direction
        dirInput = dir.normalized;
    }
    
    protected virtual void FixedUpdate()
    {
        //Apply movement on physics update
        Vector2 velocity = RB.velocity;
        Vector2 velC = dirInput * velocityChange * Time.fixedDeltaTime;

        if (velocity.x + velC.x > MaxSpeed || velocity.x + velC.x < -MaxSpeed)
            velC.x = 0.0f;

        if (velocity.y + velC.y > MaxSpeed || velocity.y + velC.y < -MaxSpeed)
            velC.y = 0.0f;
        
        RB.velocity = velocity + velC;

        if( Vector3.Distance(Vector3.zero , transform.position) > 30.0f)
        {
            Vector3 dir = (Vector3.zero - transform.position).normalized;
            RB.AddForce(dir * 20);
        }

        if(RotateTowardsDir)
        {
            if (RB)
            {
                Vector2 dir = RB.velocity.normalized;
                float angle = (Mathf.Atan2(dir.y, dir.x) * Mathf.Rad2Deg) - 90;
                Quaternion q = Quaternion.AngleAxis(angle, Vector3.forward);
                transform.rotation = Quaternion.Slerp(transform.rotation, q, Time.deltaTime * 5.0f);
            }
        }
    }
}