﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CharacterChaseState : BaseState<CharacterBehavior>
{
    Transform _target;
    Vector3 targetDirection;

    public CharacterChaseState(Transform target)
    {
        _target = target;
    }

    public override void Update(CharacterBehavior Owner)
    {
        if(_target == null)
        {
            Owner.ChangeState(new CharacterIdleState());
            return;
        }

        float dist = Vector3.Distance(Owner.transform.position, _target.position);
        if (dist < Owner.LineOfSight + 1.0f)
        {
            targetDirection = (_target.position - Owner.transform.position).normalized;
            Owner.Move(targetDirection);
            Debug.Log("Moving!");
            Debug.DrawLine(Owner.transform.position, Owner.transform.position + (targetDirection * Owner.LineOfSight), Color.cyan);
        }
        else
        {
            Owner.ChangeState(new CharacterIdleState());
        }
    }

    public override void Enter(CharacterBehavior Owner)
    {
        Owner.Anim.LookAt(_target);
    }
    public override void Exit(CharacterBehavior Owner)
    {
        Owner.Anim.LookAt(null);
        Owner.Move(Vector2.zero);
    }
}
