﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CharacterFleeState : BaseState<CharacterBehavior>
{
    Transform _target;
    Vector3 targetDirection;

    public CharacterFleeState(Transform target)
    {
        _target = target;
    }

    float eyeTimer = 0.0f;
    public override void Update(CharacterBehavior Owner)
    {
        if(_target == null)
        {
            Owner.ChangeState(new CharacterIdleState());
            return;
        }

        float dist = Vector3.Distance(Owner.transform.position, _target.position);
        if (dist < Owner.LineOfSight * 1.5f)
        {
            targetDirection = -(_target.position - Owner.transform.position).normalized;
            Owner.Move(targetDirection);
          //  Debug.DrawLine(Owner.transform.position, Owner.transform.position + (targetDirection * Owner.LineOfSight), Color.cyan);
        }
        else
        {
            Owner.ChangeState(new CharacterIdleState());
        }

        eyeTimer += Time.deltaTime;
        if (eyeTimer > 0.3f)
        {
            eyeTimer = 0.0f;
            LookForPointsOfInterest(Owner);
        }
    }

    public override void Enter(CharacterBehavior Owner)
    {
        Owner.Anim.LookAt(_target);
    }
    public override void Exit(CharacterBehavior Owner)
    {
        Owner.Anim.LookAt(null);
        Owner.Move(Vector2.zero);
    }

    void LookForPointsOfInterest(CharacterBehavior Owner)
    {
        LayerMask mask = 1 << LayerMask.NameToLayer("Collectibles");
        Collider2D[] cols = Physics2D.OverlapCircleAll(Owner.transform.position, Owner.LineOfSight, mask);
        if (cols.Length > 0)
        {
            Transform closest = cols[0].transform;
            float closestDist = (Owner.transform.position - closest.transform.position).magnitude;
            for (int i = 0; i < cols.Length; i++)
            {
                float dist = (Owner.transform.position - cols[i].transform.position).magnitude;
                if (dist < closestDist)
                {
                    if (closest == cols[i].transform)
                        continue;

                    closestDist = dist;
                    closest = cols[i].transform;
                }
            }
            Owner.ChangeState(new CharacterChaseState(closest));
            return;
        }

        mask = 1 << LayerMask.NameToLayer("Enemies");
        cols = Physics2D.OverlapCircleAll(Owner.transform.position, Owner.LineOfSight, mask);

        if (cols.Length > 0)
        {
            Transform closest = cols[0].transform;
            float closestDist = (Owner.transform.position - closest.transform.position).magnitude;
            for (int i = 0; i < cols.Length; i++)
            {
                float dist = (Owner.transform.position - cols[i].transform.position).magnitude;
                if (dist < closestDist)
                {
                    if (closest == cols[i].transform)
                        continue;

                    closestDist = dist;
                    closest = cols[i].transform;
                }
            }

            Owner.ChangeState(new CharacterFleeState(closest));
            Owner.Anim.LookAt(closest);
        }
        else
        {
            Owner.Anim.LookAt(null);
        }
    }
}
