﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CharacterIdleState : BaseState<CharacterBehavior>
{
    float eyeTimer = 0.0f;
    public override void Update(CharacterBehavior Owner)
    {
        eyeTimer += Time.deltaTime;
        if (eyeTimer > 0.3f)
        {
            eyeTimer = 0.0f;
            LookForPointsOfInterest(Owner);
        }
    }

    public override void Enter(CharacterBehavior Owner)
    {
        base.Enter(Owner);
        
        Owner.Anim.SetIdle();
    }

    void LookForPointsOfInterest(CharacterBehavior Owner)
    {
        LayerMask mask = 1 << LayerMask.NameToLayer("Collectibles");
        Collider2D[] cols = Physics2D.OverlapCircleAll(Owner.transform.position, Owner.LineOfSight, mask);
        if (cols.Length > 0)
        {
            Transform closest = cols[0].transform;
            float closestDist = (Owner.transform.position - closest.transform.position).magnitude;
            for (int i = 0; i < cols.Length; i++)
            {
                float dist = (Owner.transform.position - cols[i].transform.position).magnitude;
                if (dist < closestDist)
                {
                    if (closest == cols[i].transform)
                        continue;

                    closestDist = dist;
                    closest = cols[i].transform;
                }
            }
            Owner.ChangeState(new CharacterChaseState(closest));
            return;
        }

        mask = 1 << LayerMask.NameToLayer("Enemies");
        cols = Physics2D.OverlapCircleAll(Owner.transform.position, Owner.LineOfSight, mask);

        if (cols.Length > 0)
        {
            Transform closest = cols[0].transform;
            float closestDist = (Owner.transform.position - closest.transform.position).magnitude;
            for (int i = 0; i < cols.Length; i++)
            {
                float dist = (Owner.transform.position - cols[i].transform.position).magnitude;
                if (dist < closestDist)
                {
                    if (closest == cols[i].transform)
                        continue;

                    closestDist = dist;
                    closest = cols[i].transform;
                }
            }
            
            Owner.ChangeState(new CharacterFleeState(closest));
            Owner.Anim.LookAt(closest);
        }
        else
        {
            Owner.Anim.LookAt(null);
        }
    }
}
