﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EnemyIdleState : BaseState<EnemyBehavior>
{
    Vector2 randomVector;
    float idleTimer = 0.0f;
    float restTimer = 0.0f;

    public override void Enter(EnemyBehavior Owner)
    {
        base.Enter(Owner);
        PickRandomDirection();
        restTimer = 1.0f;
    }

    public override void Update(EnemyBehavior Owner)
    {
        if(Owner.AIType != EnemyBehavior.AITypeEnum.Passive)
            LookForPointsOfInterest(Owner);

        if (restTimer > 0.0f)
        {
            restTimer -= Time.deltaTime;
            Owner.Move(Vector2.zero);
        }
        else
        {
            Owner.Move(randomVector);
            idleTimer += Time.deltaTime;
            if(idleTimer > randT)
            {
                PickRandomDirection();
                restTimer = 3.0f;
                idleTimer = 0.0f;
            }
        }
    }

    public override void Exit(EnemyBehavior Owner)
    {
        base.Exit(Owner);
    }
    
    float randT = 0.0f;
    void PickRandomDirection()
    {
        randomVector = new Vector2(Random.value - 0.5f, Random.value - 0.5f);
        randomVector = randomVector.normalized;
        randT = Random.Range(0.5f, 5.0f);
    }

    void LookForPointsOfInterest(EnemyBehavior Owner)
    {
        LayerMask mask = 1 << LayerMask.NameToLayer("Player");
        Collider2D col = Physics2D.OverlapCircle(Owner.transform.position, Owner.LineOfSight, mask);
        
        if(col != null && col.tag == "Player")
        {
            if (Owner.AIType == EnemyBehavior.AITypeEnum.Agressive)
                Owner.ChangeState(new EnemyChaseState(col.transform.root));
            else if(Owner.AIType == EnemyBehavior.AITypeEnum.Flee)
                Owner.ChangeState(new EnemyFleeState(col.transform.root));
        }
    }

    Transform FindClosest(EnemyBehavior Owner, Collider2D[] cols)
    {
        Transform closest = cols[0].transform;
        float closestDist = (Owner.transform.position - closest.transform.position).magnitude;
        for (int i = 0; i < cols.Length; i++)
        {
            float dist = (Owner.transform.position - cols[i].transform.position).magnitude;
            if (dist < closestDist)
            {
                if (closest == cols[i].transform)
                    continue;

                closestDist = dist;
                closest = cols[i].transform;
            }
        }

        return closest;
    }
}
