﻿using UnityEngine;
using System.Collections;

public class StatePatternEnemy : MonoBehaviour 
{
	public float detectionRange = 20f;
	[HideInInspector] public MoverBehavior mover;

	public Color idleColor;
	public Color chaseColor;

	public string idleAnim;
	public string chaseAnim;

	[HideInInspector] public Transform chaseTarget;
	[HideInInspector] public IEnemyState currentState;
	[HideInInspector] public StateChase chaseState;
	[HideInInspector] public StateIdle idleState;

	private void Awake()
	{
		chaseState = new StateChase (this);
		idleState = new StateIdle (this);
		mover = GetComponent<MoverBehavior>();
	}

	// Use this for initialization
	void Start () 
	{
		//start in idle state
		currentState = idleState;
		GetComponentInChildren<SpriteRenderer>().color = idleColor;
	}

	// Update state in every frame
	void Update () 
	{
		currentState.UpdateState ();
	}

	private void OnTriggerEnter2D(Collider2D other)
	{
		currentState.OnTriggerEnter (other);
	}

	private void OnTriggerExit2D(Collider2D other)
	{
		currentState.OnTriggerExit (other);
	}
}